package com.trafficparrot.examples.jms;

import com.trafficparrot.examples.tools.jms.JmsClient;
import com.trafficparrot.examples.tools.jms.WaitForOneJmsMessage;
import org.assertj.core.api.Assertions;
import org.junit.Test;

import javax.jms.JMSException;
import java.time.Duration;
import java.time.LocalDateTime;

import static org.assertj.core.api.Assertions.assertThat;

public class HelloWorldJmsTopicIntegrationTest {
    private final JmsClient jmsClient = new JmsClient("tcp://localhost:19091");

    @Test(timeout = 30_000)
    public void helloWorldJmsTopic() throws JMSException, InterruptedException {
        WaitForOneJmsMessage waitForOneJmsMessage = jmsClient.listenOnTopic("response.topic");
        jmsClient.sendTextMessageToTopic("request.topic", "hello");

        String response = waitForOneJmsMessage.awaitMessage();

        assertThat(response).isEqualTo("world");
    }

    @Test(timeout = 30_000)
    public void helloWorldJmsTopicWithFixedDelay() throws JMSException, InterruptedException {
        WaitForOneJmsMessage waitForOneJmsMessage = jmsClient.listenOnTopic("response.topic");

        jmsClient.sendTextMessageToTopic("request.topic", "hello-delay");

        long start = System.nanoTime();
        String response = waitForOneJmsMessage.awaitMessage();
        long end = System.nanoTime();

        assertThat(response).isEqualTo("world-delay");
        assertThat(Duration.ofNanos(end - start)).isGreaterThanOrEqualTo(Duration.ofMillis(900));
    }
}
